@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 """Module for :mod:`aiida_shell.data`."""
 from .code import ShellCode
+from .pickled import PickledData
 
-__all__ = ('ShellCode',)
+__all__ = ('PickledData', 'ShellCode')
