@@ -14,7 +14,7 @@ import pathlib
 import re
 
 from aiida.engine import ExitCode
-from aiida.orm import SinglefileData
+from aiida.orm import Data, FolderData, SinglefileData
 from aiida.parsers.parser import Parser
 
 from aiida_shell.calculations.shell import ShellJob
@@ -29,11 +29,17 @@ class ShellParser(Parser):
         """Parse the contents of the output files stored in the ``retrieved`` output node."""
         dirpath = pathlib.Path(kwargs['retrieved_temporary_folder'])
 
-        missing_output_files = self.parse_custom_outputs(dirpath)
+        missing_filepaths = self.parse_custom_outputs(dirpath)
         exit_code = self.parse_default_outputs(dirpath)
 
-        if missing_output_files:
-            return self.exit_codes.ERROR_OUTPUT_FILES_MISSING.format(missing_files=', '.join(missing_output_files))
+        if 'parser' in self.node.inputs:
+            try:
+                self.call_parser_hook(dirpath)
+            except Exception as exception:  # pylint: disable=broad-except
+                return self.exit_codes.ERROR_PARSER_HOOK_EXCEPTED.format(exception=exception)
+
+        if missing_filepaths:
+            return self.exit_codes.ERROR_OUTPUT_FILEPATHS_MISSING.format(missing_filepaths=', '.join(missing_filepaths))
 
         return exit_code
 
@@ -67,13 +73,15 @@ class ShellParser(Parser):
             stderr = node_stderr.get_content()
             self.out(ShellJob.FILENAME_STDERR, node_stderr)
 
+        filename_stdout = self.node.get_option('output_filename') or ShellJob.FILENAME_STDOUT
+
         try:
-            with (dirpath / ShellJob.FILENAME_STDOUT).open(mode='rb') as handle:
-                node_stdout = SinglefileData(handle, filename=ShellJob.FILENAME_STDOUT)
+            with (dirpath / filename_stdout).open(mode='rb') as handle:
+                node_stdout = SinglefileData(handle, filename=filename_stdout)
         except FileNotFoundError:
             return self.exit_codes.ERROR_OUTPUT_STDOUT_MISSING
-        else:
-            self.out(ShellJob.FILENAME_STDOUT, node_stdout)
+
+        self.out(self.format_link_label(filename_stdout), node_stdout)
 
         try:
             exit_status = int((dirpath / ShellJob.FILENAME_STATUS).read_text())
@@ -94,25 +102,33 @@ class ShellParser(Parser):
         """Parse the output files that have been requested through the ``outputs`` input.
 
         :param dirpath: Directory containing the retrieved files.
-        :returns: List of missing output files.
+        :returns: List of missing output filepaths.
         """
         if 'outputs' not in self.node.inputs:
             return []
 
-        missing_output_files = []
+        missing_filepaths = []
 
         for filename in self.node.inputs.outputs.get_list():
-            if '*' in filename:
-                for globbed in dirpath.glob(filename):
-                    if globbed.exists():
-                        self.out(self.format_link_label(globbed.name), SinglefileData(globbed, filename=globbed.name))
-                    else:
-                        missing_output_files.append(globbed.name)
-            else:
-                filepath = dirpath / filename
-                if filepath.exists():
-                    self.out(self.format_link_label(filename), SinglefileData(filepath, filename=filename))
-                else:
-                    missing_output_files.append(filepath.name)
+            for filepath in dirpath.glob(filename) if '*' in filename else (dirpath / filename,):
+                if not filepath.exists():
+                    missing_filepaths.append(filepath.name)
+                    continue
 
-        return missing_output_files
+                if filepath.is_file():
+                    self.out(self.format_link_label(filepath.name), SinglefileData(filepath, filename=filepath.name))
+                else:
+                    self.out(self.format_link_label(filepath.name), FolderData(tree=filepath))
+
+        return missing_filepaths
+
+    def call_parser_hook(self, dirpath: pathlib.Path):
+        """Execute the ``parser`` custom parser hook that was passed as input to the ``ShellJob``."""
+        unpickled_parser = self.node.inputs.parser.load()
+        results = unpickled_parser(self, dirpath) or {}
+
+        if not isinstance(results, dict) or any(not isinstance(value, Data) for value in results.values()):
+            raise TypeError(f'{unpickled_parser} did not return a dictionary of `Data` nodes but: {results}')
+
+        for key, value in results.items():
+            self.out(key, value)
